@@ -43,8 +43,10 @@
 #include "winmonitors.h"
 #include "inputstr.h"
 #include "winclipboard/winclipboard.h"
-#include "wmutil/mouse.h"
-#include "wmutil/keyboard.h"
+
+#ifndef WM_GETDPISCALEDSIZE
+#define WM_GETDPISCALEDSIZE 0x02E4
+#endif
 
 /*
  * Global variables
@@ -142,10 +144,16 @@ winWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
         }
         return 0;
 
+    case WM_GETDPISCALEDSIZE:
+        winDebug("winTopLevelWindowProc - WM_GETDPISCALEDSIZE\n");
+        // We don't change the lParam SIZE, so the Window retains the same size
+        // in pixels (rather than getting linearly scaled by the dpi value)
+        return TRUE;
+
     case WM_DISPLAYCHANGE:
         /*
            WM_DISPLAYCHANGE seems to be sent when the monitor layout or
-           any monitor's resolution or depth changes, but it's lParam and
+           any monitor's resolution or depth changes, but its lParam and
            wParam always indicate the resolution and bpp for the primary
            monitor (so ignore that as we could be on any monitor...)
          */
@@ -153,7 +161,7 @@ winWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
         /* We cannot handle a display mode change during initialization */
         if (s_pScreenInfo == NULL)
             FatalError("winWindowProc - WM_DISPLAYCHANGE - The display "
-                       "mode changed while we were intializing.  This is "
+                       "mode changed while we were initializing.  This is "
                        "very bad and unexpected.  Exiting.\n");
 
         /*
@@ -255,6 +263,8 @@ winWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
                         }
                 }
 
+                winUpdateDpi();
+
                 /*
                    XXX: probably a small bug here: we don't compute the work area
                    and allow for task bar
@@ -263,10 +273,8 @@ winWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
                    the server is started
                  */
 
-                /* Set screen size to match new size, if it is different to current */
-                if (((dwWidth != 0) && (dwHeight != 0)) &&
-                    ((s_pScreenInfo->dwWidth != dwWidth) ||
-                     (s_pScreenInfo->dwHeight != dwHeight))) {
+                /* Set screen size to match new size */
+                if ((dwWidth != 0) && (dwHeight != 0)) {
                     winDoRandRScreenSetSize(s_pScreen,
                                             dwWidth,
                                             dwHeight,
@@ -853,7 +861,7 @@ winWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
         switch (wParam) {
         case WIN_E3B_TIMER_ID:
             /* Send delayed button press */
-            winMouseButtonsSendEvent(TRUE,
+            winMouseButtonsSendEvent(ButtonPress,
                                      s_pScreenPriv->iE3BCachedPress);
 
             /* Kill this timer */
